#! /opt/bin/dash
#
# os/prog/checksum/post.sh

export PATH="/opt/bin:/bin:/usr/bin"

echo "Content-type: text/html\n\n<html><head><title>checksum</title></head><body>"
user=`echo "${REMOTE_USER}" | tr -dc '[:alnum:]' | tr '[:upper:]' '[:lower:]'`
if [ -z "$user" ]; then echo "Bad user: ${REMOTE_USER} -&gt; $user"; exit 1; fi
if [ -n "$HTTP_HOST" ]; then cd /os/prog/checksum || exit 1; fi

# tomato = #FF6347
# lightgreen = #90EE90
#
in=$(cat | egrep -v '#FF6347|#90EE90')
h=$(echo "$in" | awk '{ if( $2 == "P") { print $3; exit; }}')
useed=$(echo "$in" | awk '{ if( $2 == "useed") { gsub(/[^0-9]/,"",$3); print $3; exit; }}')

#echo "<pre>$in\n</pre>"

case "$h" in
 "2") p="2"; s="123456789";;
   *) p="1"; s="123456789";;
esac

if [ -n "$useed" ]; then
  seed="$useed"
  msg="- seed = $seed"
else
  x=$(perl -e "print unpack(\"C*\",$user);"); seed="${s}${x}"
  #echo "<pre>s = $s, x = $x, seed = $seed</pre>"
  msg=""
fi

echo "<b>$user</b> - checksum #$p results $msg<p>
<table border=0><tr><td bgcolor=\"#90EE90\">green = correct</td></tr>
<tr><td bgcolor=\"#FF6347\"> &nbsp; &nbsp; red = wrong</td></tr></table><p>"

out=$(
(echo "$in"; python3 ./checksum.py -s "$seed" -c) | awk '
 function hex(h) { h=substr(h,3); # remove leading 0x
  while(length(h)>2 && index(h,"0") == 1) h=substr(h,2); # remove excess leading zeros
  while(length(h)<2) h = "0"h; return "0x"h; } # add leading zeros to make length 2
 function bin(h) { h=substr(h,3); # remove leading 0b
  while(length(h)>8 && index(h,"0") == 1) h=substr(h,2); # remove excess leading zeros
  while(length(h)<8) h = "0"h; return "0b"h; } # add leading zeros to make length 8
 function bits(d,r) { r=""; # convert to 8-bit binary string
  for(i=0;i<8;++i) { r=(and(d,1)?"1":"0")""r; d=rshift(d,1); } return "0b"r; }
 BEGIN { print "\n<table border=0>"; }
 { if( $1 == "") next;
   if( $1 == "U") { if( index($3,"0x") == 1) u[$2]=hex($3);
    else if( index($3,"0b") == 1) u[$2]=bin($3); else u[$2]=$3;
   # print "<pre>"$2, u[$2]"</pre>";
   } else {
    if( $1 == "Add:") { d=$2; x=sprintf("0x%02x",d); b=bits(d);
     v=u["A"]; if( v == d || v == x || v == b) color="#90EE90"; else color="#FF6347";
     if( v == "") v = "&nbsp;"
     print "<tr><td align=left>Add: </td><td align=left bgcolor=\""color"\"> "v"</td></tr>";
    } else if( $1 == "Xor:") { d=$2; x=sprintf("0x%02x",d); b=bits(d);
     v=u["X"]; if( v == d || v == x || v == b) color="#90EE90"; else color="#FF6347";
     if( v == "") v = "&nbsp;"
     print "<tr><td align=left>Xor: </td><td align=left bgcolor=\""color"\"> "v"</td></tr>";
    } else if( $1 == "Fletcher(a,b):") { gsub(/[\(\),]/," ",$0); d1=$5; d2=$6;
     x1=sprintf("0x%02x",d1); b1=bits(d1); x2=sprintf("0x%02x",d2); b2=bits(d2);
     v1=u["F1"]; if( v1 == d1 || v1 == x1 || v1 == b1) color="#90EE90"; else color="#FF6347";
     if( v1 == "") v1 = "&nbsp;"
     print "<tr><td align=left>Fletcher1: </td><td align=left bgcolor=\""color"\"> "v1"</td></tr>";
     v2=u["F2"]; if( v2 == d2 || v2 == x2 || v2 == b2) color="#90EE90"; else color="#FF6347";
     if( v2 == "") v2 = "&nbsp;"
     print "<tr><td align=left>Fletcher2: </td><td align=left bgcolor=\""color"\"> "v2"</td></tr>";
     print "</table>";
    } 
  } 
}'
)

# Add:            196       (0b11000100)
# Xor:            134       (0b10000110)
# Fletcher(a,b):  198,223   (0b11000110,0b11011111)
# after gsub:
# Fletcher a b :  198 223    0b11000110 0b11011111

if [ -z "$useed" ]; then
  bad=$(echo "$out" | grep "#FF6347" | wc -l)
  good=$(echo "$out" | grep "#90EE90" | wc -l)
  log=$(echo "$in" | tr -dc '\-+,:[:alnum:]')
  echo "$(date) $user ${p}: good $good bad $bad in: $log" >> log
fi

echo "<pre>\n$out</pre></body></html>"
