#! /opt/bin/dash
#
# os/prog/fsck/post.sh

export PATH="/opt/bin:/bin:/usr/bin"

echo "Content-type: text/html\n\n<html><head><title>fsck</title></head><body>"
user=`echo "${REMOTE_USER}" | tr -dc '[:alnum:]' | tr '[:upper:]' '[:lower:]'`
if [ -z "$user" ]; then echo "Bad user: ${REMOTE_USER} -&gt; $user"; exit 1; fi
if [ -n "$HTTP_HOST" ]; then cd /os/prog/fsck || exit 1; fi

# tomato = #FF6347
# lightgreen = #90EE90
#
in=$(cat | egrep -v '#FF6347|#90EE90')
h=$(echo "$in" | awk '{ if( $2 == "P") { print $3; exit; }}')
useed1=$(echo "$in" | awk '{ if( $2 == "useed1") { gsub(/[^0-9]/,"",$3); print $3; exit; }}')
useed2=$(echo "$in" | awk '{ if( $2 == "useed2") { gsub(/[^0-9]/,"",$3); print $3; exit; }}')
if [ -z "$useed2" ]; then useed2="$useed1"; fi

#echo "<pre>$in\n</pre>"

n="6"; i="8"; d="8"

case "$h" in
 "3") p="3"; s1="123456789"; s2="123456789";;
 "2") p="2"; s1="123456789"; s2="123456789";;
   *) p="1"; s1="123456789"; s2="123456789";;
esac

if [ -n "$useed1" ]; then
  seed1="$useed1"
  seed2="$useed2"
  msg="- seed1 = $seed1, seed2 = $seed2"
else
  x=$(perl -e "print unpack(\"C*\",$user);"); seed1="${s1}${x}"; seed2="${s2}${x}"
  #echo "<pre>s1 = $s1, s2 = $s2, x = $x, seed1 = $seed1, seed2 = $seed2</pre>"
  msg=""
fi

echo "<b>$user</b> - fsck #$p results $msg<p>
<table border=0><tr><td bgcolor=\"#90EE90\">green = correct</td></tr>
<tr><td bgcolor=\"#FF6347\"> &nbsp; &nbsp; red = wrong</td></tr></table>
<p><b>${p}. How is the file system corrupted?</b><p>"

# for -w option: 0...11 skipping 5,7,11
w=$(awk -v "S=$seed2" 'END { len=length(S); s=0;
  for(i=1;i<len;i+=2) s=and(s*31+substr(S,i,2),0xffffffff); # 32-bit hashcode
  n=split("0,1,2,3,4,6,8,9,10",a,","); srand(s); k=1+int(rand()*n); print a[k];
}' < /dev/null)

# 0    CORRUPTION::DATA BITMAP corrupt bit 12
# 1    CORRUPTION::INODE BITMAP corrupt bit 12
# 2,8  CORRUPTION::INODE 14 refcnt increased
# 3,9  CORRUPTION::INODE 11 orphan
# 4,10 CORRUPTION::INODE 14 points to dead block 5
# 6    CORRUPTION::INODE 8 with directory [('.', 8), ('..', 0)]:

out=$(
(echo "$in"; python3 ./fsck.py -s "$seed1" -S "$seed2" -w "$w" -n "$n" -i "$i" -d "$d" -c) |
  awk -v "c=$w" '{ if( $1 != "") { if( $1 == "U") u[$2]=$3;
   else if($1 ~ /CORRUPTION/) {
      if(c == 8) c=2; else if( c == 9) c=3; else if(c == 10) c=4;
      if(c == 0 || c == 1) v=$5; else v=$2;
      x=u["r"]; if( x == "v"c"v") xcolor="#90EE90"; else xcolor="#FF6347";
      y=u["v"]; if( y == v) ycolor="#90EE90"; else ycolor="#FF6347";
      if(x=="v0v") xmsg="DATA BITMAP corrupt bit";
      else if(x=="v1v") xmsg="INODE BITMAP corrupt bit";
      else if(x=="v2v") xmsg="INODE refcnt wrong";
      else if(x=="v3v") xmsg="INODE orphan";
      else if(x=="v4v") xmsg="INODE points to dead block";
      else if(x=="v6v") xmsg="INODE dir entry refers to unallocated inode";
      else xmsg=" &nbsp; ";
      print "<table border=0>";
       print "<tr><td align=left bgcolor=\""xcolor"\">"xmsg" &nbsp; </td></tr>";
       print "<tr><td align=left bgcolor=\""ycolor"\">INODE or bit number: "y" &nbsp; </td></tr>";
      print "</table>";
    exit;
   }
  }}'
)

if [ -z "$useed1" ]; then
  bad=$(echo "$out" | grep "#FF6347" | wc -l)
  good=$(echo "$out" | grep "#90EE90" | wc -l)
  log=$(echo "$in" | tr -dc '[:alnum:]')
  echo "$(date) $user ${p}: good $good bad $bad in: $log" >> log
fi

echo "<pre>\n$out</pre></body></html>"
